% This function gets the critical value for the sequential test SEQ(k+1|k) in Kejriwal and Perron (2010).
% Input:
%   k: the number of breaks
%   p: number of I(0) variables (not include the intercept)
%   q: number of I(1) variables
%   opt_tr: =0, nontredning case, =1 trending case 
%   eps_trim: trimming value, 0.15 is the default level and the only option
%   siglev: significance level
%
% Output:
%   cv_sqk: corresponding critical values 
function cv_sqk = get_SEQcv(k,p,q,opt_tr,eps_trim,siglev)

if p == 0 && q == 1
cv = ...
[           12        12.94        13.74        14.53        15.23        12.94        13.99        14.93         15.5        15.73;
        13.78        15.25        16.38        17.02         17.7        15.01        15.85        16.53        16.86        17.04;
        16.38         17.7        18.24        18.53        19.18        16.53        17.04        17.17        17.43        18.04;
        18.53        19.33        19.92         20.5        21.34        17.43        18.58        19.11        19.22        19.54];
		
elseif p == 0 && q == 2
cv = ...
[        14.26        15.02        15.64        16.02        16.51        13.57        14.78         15.4        15.87        16.12;
        15.65        16.61        17.12        17.66        17.85        15.51        16.18        17.08        17.31         17.5;
        17.12        17.85        18.22        19.04        19.27        17.08         17.5        19.27        19.62         19.7;
        19.04        19.35         19.9        19.99        20.01        19.62        19.79        21.52        22.58        22.75];

elseif p == 0 && q == 3
cv = ...
[        16.64        17.57        18.28        18.86        19.53        16.38         17.3        17.92         18.4        18.62;
         18.3        19.58        20.21        20.77        21.45        17.99        18.74        19.77        20.28        20.89;
        20.21        21.45        22.67        23.36        23.48        19.77        20.89        21.56        22.11        22.28;
        23.36        23.52        24.13        24.43        25.16        22.11        22.37        22.83        23.98        24.54];

elseif p == 0 && q == 4
cv = ...
[        18.96        19.91        20.68        21.13        21.51        18.29        19.54        20.43        20.97        21.32;
         20.8        21.59        22.36        22.58        23.12        20.51        21.81         22.4        23.12        23.78;
        22.36        23.12         24.1        25.73        26.11         22.4        23.78         25.1        25.75        25.84;
        25.73        27.01        27.43        27.47        27.75        25.75        26.36        26.66        26.86        27.71];

elseif p == 1 && q == 1
cv = ...
[        13.18        13.92         14.7        15.08        15.79        13.72        15.14        15.72        16.44        16.75;
        14.72        15.82         16.6        17.28        17.61        15.73        16.83        17.54        17.99        18.17;
         16.6        17.61         19.2        19.43        19.85        17.54        18.17        19.27        19.97        20.53;
        19.43        20.02        21.38        21.43         22.1        19.97        21.13        22.77        23.42        23.98];
		
elseif p == 2 && q == 1
cv = ...
[        15.06        16.32        17.39        17.83        18.22        15.09        16.21        16.85        17.33        17.85;
        17.44        18.25        18.65         19.1        19.96        16.86        17.87        18.81        18.95        19.28;
        18.65        19.96        20.06        20.37        20.69        18.81        19.28        19.66         21.1        21.43;
        20.37        20.73        21.96        23.13        23.22         21.1        21.61        22.74         23.7        24.12];
		
elseif p == 1 && q == 2
cv = ...
[        15.82        16.69        17.59        18.15        18.39        15.21        16.54        17.44        17.98        18.46;
        17.68        18.63        19.37        19.89        20.39        17.49        18.49        19.26        19.61        20.27;
        19.37        20.39        21.48        22.63        22.84        19.26        20.27        20.76        21.69        22.03;
        22.63        23.82        24.73         25.4        25.62        21.69        22.37        22.94        24.08        24.08];

elseif p == 2 && q == 2
cv = ...
[        16.95        18.69        19.46        20.06        20.44        17.12        18.56         19.4        19.92        20.75;
        19.48        20.44        21.33        21.66        21.97        19.45        20.42        21.16        21.46        22.33;
        21.33        21.97        22.39        23.52        24.03        21.16        21.86        22.89        23.41        23.85;
        23.52        24.11        24.75        25.05        25.12        23.41        23.85        25.06        25.94        26.32];
end

if eps_trim ~= 0.15
disp('Warning: cv with this trimming cannot be found in this program. The default is 0.15 trimming. The corresponding result might be wrong.');
end

if siglev == 0.1 indsig = 1;
elseif siglev == 0.05 indsig = 2;
elseif siglev == 0.025 indsig = 3;
elseif siglev == 0.01 indsig = 4;
else disp('Warning: cv with this siglev cannot be found in this program. The corresponding result might be wrong.'); indsig = 2;
end

cv_sqk = cv(indsig,opt_tr*5+k);
end